*********************************************************************************************************************************************;
*** NAACCR Rural Urban Linkage Program - VERSION 09/27/2018
/*SAS program to create:
   - Rural Urban Continuum Codes for 1993, 2003, and 2013 based on the state and county
   - Rural Urban Commuting Area Codes for 2000 and 2010 based on the state, county, and census tract
   - Urban Rural Census Indicator for 2000 and 2010 based on the state, county, and census tract

  Name of file containing poverty group information by state, county, and census tract:
         rural_urban_format_lib.fmt

*********************************************************************************************************************************************;
**** Written/modified:                                                                                                                       ;
****    Andrew Lake, Information Management Services, Inc., 301.984.3445                                                                     ;
*********************************************************************************************************************************************;

*********************************************************************************************************************************************;
/***** NAACCR Version 16.0 input registry file, with the following fields included:
       Patient ID [item #20]
       State at DX [item #80]
       County at DX [item #90]
       Census Tract 2000 [item #130]
       Census Tract 2010 [item #135]
****/
filename rurb 'rural_urban_linkage.fmt';
%include rurb;

filename regfile './input.txt';

**** Output file Name - Text file with rural urban fields added;
filename outfile './output.txt';
*********************************************************************************************************************************************;

%let VALID_STATES = "AL" "AK" "AZ" "AR" "CA" "CO" "CT" "DE" "DC" "FL" "GA" "HI" "ID" "IL" "IN" "IA" "KS" 
                    "KY" "LA" "ME" "MD" "MA" "MI" "MN" "MS" "MO" "MT" "NE" "NV" "NH" "NJ" "NM" "NY" "NC" 
                    "ND" "OH" "OK" "OR" "PA" "RI" "SC" "SD" "TN" "TX" "UT" "VT" "VA" "WA" "WV" "WI" "WY" 
                    "AS" "GU" "MP" "PW" "PR" "UM" "VI" "FM" "MH" "TT" "AB" "BC" "MB" "NB" "NL" "NS" "NT" 
                    "NU" "ON" "PE" "QC" "SK" "YT" "AA" "AE" "AP";
%let miss_unk_STATES = " " "CD" "US" "XX" "YY" "ZZ";
%let miss_unk_COUNTIES = " " "999";
%let miss_unk_CENSUS_TRACTS = " " "000000" "999999";


 /********************************************************************************************************************************************
   THIS DATA STEP....
   
     * Calculates the 1993, 2003, and 2013 rural urban continuum codes for the provided record using the following variables:
     * -- addressAtDxState (#80)
     * -- addressAtDxCounty (#90)
     * The returned urban rural continuum code will have one of the following values:
     * -- 00 = Central counties of metro areas of 1 million population or more (1993 rural urban continuum code only)
     * -- 01 = Counties of metro areas of 1 million population or more
     * -- 02 = Counties in metro areas of 250,000 to 1 million population
     * -- 03 = Counties in metro areas of fewer than 250,000 population
     * -- 04 = Urban population of 20,000 or more, adjacent to a metro area
     * -- 05 = Urban population of 20,000 or more, not adjacent to a metro area
     * -- 06 = Urban population of 2,500 to 19,999, adjacent to a metro area
     * -- 07 = Urban population of 2,500 to 19,999, not adjacent to a metro area
     * -- 08 = Completely rural or less than 2,500 urban population, adjacent to a metro area
     * -- 09 = Completely rural or less than 2,500 urban population, not adjacent to a metro area
     * -- 96 = Invalid state abbreviation or county code
     * -- 97 = Insufficient population data for county to determine rural urban code
     * -- 98 = Valid state/county combination but rural urban code not found
     * -- 99 = Missing or unknown state or county

     * Calculates the 2000 and 2010 rural urban commuting area codes for the provided record using the following variables:
     * -- addressAtDxState (#80)
     * -- addressAtDxCounty (#90)
     * -- censusct2000 (#130)
     * -- censusct2010 (#135)
     * The returned urban rural commuting area will have one of the following values:
     * -- 1 = Urban commuting area - Secondary RUCA code is 1.0, 1.1, 2.0, 2.1, 3.0, 4.1, 5.1, 7.1, 8.1, or 10.1
     * -- 2 = Not an urban commuting area - all other RUCA codes except 99
     * -- 9 = Primary RUCA code is 99 (valid unknown)
     * -- A = Invalid state abbreviation, county code, or census tract
     * -- B = Insufficient population data for county to determine RUCA code
     * -- C = Valid state/county/tract combination but RUCA code not found
     * -- D = Missing or unknown state, county, or census tract

     * Calculates the 2000 and 2010 rural urban census indicators for the provided record using the following variables:
     * -- addressAtDxState (#80)
     * -- addressAtDxCounty (#90)
     * -- censusct2000 (#130)
     * -- censusct2010 (#135)
     * The returned urban rural census indicator will have one of the following values:
     * -- 1 = All urban - the percentage of the population in an urban area is 100%
     * -- 2 = Mostly urban - the percentage of the population in an urban area is between >=50% and <100%
     * -- 3 = Mostly rural - the percentage of the population in an urban area is between >0% and <50%
     * -- 4 = All rural - the percentage of the population in an urban area is 0%
     * -- 9 = The percentage of the population in an urban or rural area is unknown (valid unknown)
     * -- A = Invalid state, county code, or census tract
     * -- B = Insufficient population data for county to determine census indicator
     * -- C = Valid state/county/tract combination but census indicator not found
     * -- D = Missing or unknown state, county, or census tract
     */

data registry_data (drop=rec_state);
  /* read in cancer registry file with selected fields-based on NAACCR Version 16.0 file*/
  infile regfile lrecl=3339 pad;
  input  @001 record    $char3339.
         @145 state     $char2.
         @156 county    $char3.
         @168 ct2000    $char6.
         @428 ct2010    $char6.;
         
  length rucc93 rucc03 rucc13 $2.;
  length ruca2000 ruca2010 ur2000 ur2010 $1.;
  
  *** RECODE SEER CITIES FOR LOOKUP ***;
       if upcase(state) = "AT" then rec_state = "GA";
  else if upcase(state) = "DT" then rec_state = "MI";
  else if upcase(state) = "GB" then rec_state = "CA";
  else if upcase(state) = "LO" then rec_state = "CA";
  else if upcase(state) = "SE" then rec_state = "WA";
  else rec_state = upcase(state);
         
  if rec_state in (&VALID_STATES.) then state_is_valid = 1;
  else state_is_valid = 0;
  
  if rec_state in (&MISS_UNK_STATES.) then state_is_miss_unk = 1;
  else state_is_miss_unk = 0;
  
  if state_is_valid = 1 or state_is_miss_unk = 1 then state_is_val_miss_unk = 1;
  else state_is_val_miss_unk = 0;
  
  if notdigit(county) = 0 and lengthn(trim(county)) = 3 then county_is_valid = 1;
  else county_is_valid = 0;
  
  if county in (&MISS_UNK_COUNTIES.) then county_is_miss_unk = 1;
  else county_is_miss_unk = 0;
  
  if county_is_valid = 1 or county_is_miss_unk = 1 then county_is_val_miss_unk = 1;
  else county_is_val_miss_unk = 0;
  
  if notdigit(ct2000) = 0 and lengthn(trim(ct2000)) = 6 then ct2000_is_valid = 1;
  else ct2000_is_valid = 0;
  
  if ct2000 in (&MISS_UNK_CENSUS_TRACTS.) then ct2000_is_miss_unk = 1;
  else ct2000_is_miss_unk = 0;
  
  if ct2000_is_valid = 1 or ct2000_is_miss_unk = 1 then ct2000_is_val_miss_unk = 1;
  else ct2000_is_val_miss_unk = 0;  

  if notdigit(ct2010) = 0 and lengthn(trim(ct2010)) = 6 then ct2010_is_valid = 1;
  else ct2010_is_valid = 0;
  
  if ct2010 in (&MISS_UNK_CENSUS_TRACTS.) then ct2010_is_miss_unk = 1;
  else ct2010_is_miss_unk = 0;
  
  if ct2010_is_valid = 1 or ct2010_is_miss_unk = 1 then ct2010_is_val_miss_unk = 1;
  else ct2010_is_val_miss_unk = 0; 

  *** GET RURAL URBAN CONTINUUM CODE ***;
  rucc93 = "98";
  rucc03 = "98";
  rucc13 = "98";  
  if (state_is_val_miss_unk = 0) or (county_is_val_miss_unk = 0) then do;
    rucc93 = "96";
    rucc03 = "96";
    rucc13 = "96";
  end;
  else if (state_is_miss_unk = 1) or (county_is_miss_unk = 1) then do;
    rucc93 = "99";
    rucc03 = "99";
    rucc13 = "99";
  end;
  else if (county = "000") then do;
    rucc93 = "97";
    rucc03 = "97";
    rucc13 = "97";
  end;
  else do;
    rucc93 = put(rec_state||county,rucc93f.);
    rucc03 = put(rec_state||county,rucc03f.);
    rucc13 = put(rec_state||county,rucc13f.);
  end;
  
  *** GET 2000 RURAL URBAN COMMUTING AREA, 2000 Urban Rural Census Indicator ***;
  ruca2000 = "C";
  ur2000 = "C";
  if (state_is_val_miss_unk = 0) or (county_is_val_miss_unk = 0) or (ct2000_is_val_miss_unk = 0) then do;
    ruca2000 = "A";
    ur2000 = "A";
  end;
  else if (state_is_miss_unk = 1) or (county_is_miss_unk = 1) or (ct2000_is_miss_unk = 1) then do;
    ruca2000 = "D";
    ur2000 = "D";
  end;
  else if (county = "000") then do;
    ruca2000 = "B";
    ur2000 = "B";
  end;
  else do;
    ruca2000 = put(rec_state||county||ct2000,ruca2000f.);
    ur2000 = put(rec_state||county||ct2000,ur2000f.);
  end;

  *** GET 2010 RURAL URBAN COMMUTING AREA, 2010 Urban Rural Census Indicator ***;
  ruca2010 = "C";
  ur2010 = "C";
  if (state_is_val_miss_unk = 0) or (county_is_val_miss_unk = 0) or (ct2010_is_val_miss_unk = 0) then do;
    ruca2010 = "A";
    ur2010 = "A";
  end;
  else if (state_is_miss_unk = 1) or (county_is_miss_unk = 1) or (ct2010_is_miss_unk = 1) then do;
    ruca2010 = "D";
    ur2010 = "D";
  end;
  else if (county = "000") then do;
    ruca2010 = "B";
    ur2010 = "B";
  end;
  else do;
    ruca2010 = put(rec_state||county||ct2010,ruca2010f.);
    ur2010 = put(rec_state||county||ct2010,ur2010f.);
  end;

  label record   = 'Incidence Record'
        state    = 'Addr at DX - State'
        county   = 'County at DX'
        ct2000   = 'Census tract 2000'
        ct2010   = 'Census tract 2010'
        rucc93   = '1993 Rural Urban Continuum Code'
        rucc03   = '2003 Rural Urban Continuum Code'
        rucc13   = '2013 Rural Urban Continuum Code'
        ruca2000 = '2000 Rural Urban Commuting Area'
        ruca2010 = '2010 Rural Urban Commuting Area'
        ur2000   = '2000 Urban Rural Census Indicator'
        ur2010   = '2010 Urban Rural Census Indicator'
        ;
run;

/* Create a new file with state, county, percent poverty variables - Census Tract information (Items #130, #135) is not included*/
data _null_;
  set registry_data;
  file outfile lrecl=3339 pad;
  put @001  record $char3339.
      @168  '      '
      @428  '      '
      @424  rucc93
      @426  rucc03
      @476  rucc13
      @2347 ruca2000
      @2348 ruca2010
      @2349 ur2000
      @2350 ur2010
      ;
  run;
  
proc format;
 value $rucc
     "00" = "00: Central counties of metro areas of 1 million population or more (1993 rural urban continuum code only)"
     "01" = "01: Counties of metro areas of 1 million population or more"
     "02" = "02: Counties in metro areas of 250,000 to 1 million population"
     "03" = "03: Counties in metro areas of fewer than 250,000 population"
     "04" = "04: Urban population of 20,000 or more, adjacent to a metro area"
     "05" = "05: Urban population of 20,000 or more, not adjacent to a metro area"
     "06" = "06: Urban population of 2,500 to 19,999, adjacent to a metro area"
     "07" = "07: Urban population of 2,500 to 19,999, not adjacent to a metro area"
     "08" = "08: Completely rural or less than 2,500 urban population, adjacent to a metro area"
     "09" = "09: Completely rural or less than 2,500 urban population, not adjacent to a metro area"
     "96" = "96: Invalid state abbreviation or county code"
     "97" = "97: Insufficient population data for county to determine rural urban code"
     "98" = "98: Valid state/county combination but rural urban code not found"
     "99" = "99: Missing or unknown state or county";

 value $ruca
     "1" = "1: Urban commuting area - Secondary RUCA code is 1.0, 1.1, 2.0, 2.1, 3.0, 4.1, 5.1, 7.1, 8.1, or 10.1"
     "2" = "2: Not an urban commuting area - all other RUCA codes except 99"
     "9" = "9: Primary RUCA code is 99 (valid unknown)"
     "A" = "A: Invalid state abbreviation, county code, or census tract"
     "B" = "B: Insufficient population data for county to determine RUCA code"
     "C" = "C: Valid state/county/tract combination but RUCA code not found"
     "D" = "D: Missing or unknown state, county, or census tract";

 value $urcensus
     "1" = "1: All urban - the percentage of the population in an urban area is 100%  "
     "2" = "2: Mostly urban - the percentage of the population in an urban area is between >=50% and <100%"
     "3" = "3: Mostly rural - the percentage of the population in an urban area is between >0% and <50%"
     "4" = "4: All rural - the percentage of the population in an urban area is 0%"
     "9" = "9: The percentage of the population in an urban or rural area is unknown (valid unknown)"
     "A" = "A: Invalid state, county code, or census tract"
     "B" = "B: Insufficient population data for county to determine census indicator"
     "C" = "C: Valid state/county/tract combination but census indicator not found"
     "D" = "D: Missing or unknown state, county, or census tract";
run;

options center;
%let path = %sysfunc(getoption(sysin));
footnote "&path.";

title "Frequency of Rural-Urban Variables";
proc freq data = registry_data;
  tables rucc93 / nocum;
  tables rucc03 / nocum;
  tables rucc13 / nocum;
  tables ruca2000 / nocum;
  tables ruca2010 / nocum;
  tables ur2000 / nocum;
  tables ur2010 / nocum;
  format rucc93 rucc03 rucc13 $rucc. ruca2000 ruca2010 $ruca. ur2000 ur2010 $urcensus.;
run;

title "State/County Combinations of Records with 1993 Rural Urban Continuum Code Not Found";
proc freq data = registry_data;
  tables state * county * rucc93 / missing list nocum;
  where rucc93 in ("96" "97" "98" "99");
  format rucc93 $rucc.;
run;

title "State/County Combinations of Records with 2003 Rural Urban Continuum Code Not Found";
proc freq data = registry_data;
  tables state * county * rucc03 / missing list nocum;
  where rucc03 in ("96" "97" "98" "99");
  format rucc03 $rucc.;
run;

title "State/County Combinations of Records with 2013 Rural Urban Continuum Code Not Found";
proc freq data = registry_data;
  tables state * county * rucc13 / missing list nocum;
  where rucc13 in ("96" "97" "98" "99");
  format rucc13 $rucc.;
run;

title "State/County/Census Tract 2000 Combinations of Records with 2000 Rural Urban Commuting Area Not Found";
proc freq data = registry_data;
  tables state * county * ct2000 * ruca2000 / missing list nocum;
  where ruca2000 in ("A" "B" "C" "D");
  format ruca2000 $ruca.;
run;

title "State/County/Census Tract 2010 Combinations of Records with 2010 Rural Urban Commuting Area Not Found";
proc freq data = registry_data;
  tables state * county * ct2010 * ruca2010 / missing list nocum;
  where ruca2010 in ("A" "B" "C" "D");
  format ruca2010 $ruca.;
run;

title "State/County/Census Tract 2000 Combinations of Records with 2000 Urban Rural Indicator (Census) Not Found";
proc freq data = registry_data;
  tables state * county * ct2000 * ur2000 / missing list nocum;
  where ur2000 in ("A" "B" "C" "D");
  format ur2000 $urcensus.;
run;

title "State/County/Census Tract 2010 Combinations of Records with 2010 Urban Rural Indicator (Census) Not Found";
proc freq data = registry_data;
  tables state * county * ct2010 * ur2010 / missing list nocum;
  where ur2010 in ("A" "B" "C" "D");
  format ur2010 $urcensus.;
run;